@@ -31,6 +31,8 @@ typedef struct _AlpmDB {
   pmdb_t *c_data;
 } AlpmDB;
 
+static PyTypeObject AlpmDBType;
+
 static void pyalpm_db_dealloc(AlpmDB *self) {
   Py_TYPE(self)->tp_free((PyObject*)self);
 }
@@ -48,38 +50,44 @@ static PyObject* _pyobject_from_pmgrp(void *group) {
   return tuple;
 }
 
+/** Converts a Python list of databases to an alpm_list_t linked list.
+ * return 0 on success, -1 on failure
+ */
+int pylist_db_to_alpmlist(PyObject *list, alpm_list_t **result) {
+  alpm_list_t *ret = NULL;
+  PyObject *iterator = PyObject_GetIter(list);
+  PyObject *item;
+
+  if(iterator == NULL) {
+    PyErr_SetString(PyExc_TypeError, "object is not iterable");
+    return -1;
+  }
+
+  while((item = PyIter_Next(iterator)))
+  {
+    if (PyObject_TypeCheck(item, &AlpmDBType)) {
+      ret = alpm_list_add(ret, ((AlpmDB*)item)->c_data);
+    } else {
+      PyErr_SetString(PyExc_TypeError, "list must contain only Database objects");
+      FREELIST(ret);
+      Py_DECREF(item);
+      Py_DECREF(iterator);
+      return -1;
+    }
+    Py_DECREF(item);
+  }
+  Py_DECREF(iterator);
+
+  *result = ret;
+  return 0;
+}
+
 #define CHECK_IF_INITIALIZED() if (! self->c_data) { \
   PyErr_SetString(alpm_error, "data is not initialized"); \
   return NULL; \
   }
 
-static PyObject* pyalpm_db_get_pkg(PyObject *rawself, PyObject* args) {
-  char *pkgname;
-  pmpkg_t *p;
-  AlpmDB *self = (AlpmDB*)rawself;
-
-  if(!PyArg_ParseTuple(args, "s", &pkgname))
-  {
-    PyErr_SetString(PyExc_TypeError, "get_pkg() takes a string argument");
-    return NULL;
-  }
-
-  CHECK_IF_INITIALIZED();
-
-  p = alpm_db_get_pkg(self->c_data, pkgname);
-
-  if (p == NULL) {
-    Py_RETURN_NONE;
-  } else {
-    PyObject *result;
-    result = pyalpm_package_from_pmpkg(p);
-    if (result == NULL) {
-      return NULL;
-    } else {
-      return result;
-    }
-  }
-}
+/** Database properties */
 
 static PyObject* pyalpm_db_get_name(AlpmDB* self, void* closure) {
   CHECK_IF_INITIALIZED();
@@ -129,6 +137,53 @@ static PyObject* pyalpm_db_get_grpcache(AlpmDB* self, void* closure) {
   return alpmlist_to_pylist(grplist, _pyobject_from_pmgrp);
 }
 
+/** Package get/set operations */
+
+static PyObject* pyalpm_db_get_pkg(PyObject *rawself, PyObject* args) {
+  char *pkgname;
+  pmpkg_t *p;
+  AlpmDB *self = (AlpmDB*)rawself;
+
+  if(!PyArg_ParseTuple(args, "s", &pkgname))
+  {
+    PyErr_SetString(PyExc_TypeError, "get_pkg() takes a string argument");
+    return NULL;
+  }
+
+  CHECK_IF_INITIALIZED();
+
+  p = alpm_db_get_pkg(self->c_data, pkgname);
+
+  if (p == NULL) {
+    Py_RETURN_NONE;
+  } else {
+    PyObject *result;
+    result = pyalpm_package_from_pmpkg(p);
+    if (result == NULL) {
+      return NULL;
+    } else {
+      return result;
+    }
+  }
+}
+
+static PyObject* pyalpm_db_set_pkgreason(PyObject* rawself, PyObject* args) {
+  char *pkgname;
+  int reason;
+  AlpmDB *self = (AlpmDB*)rawself;
+  int ret;
+  if (!PyArg_ParseTuple(args, "si", &pkgname, &reason)) {
+    PyErr_SetString(PyExc_TypeError, "expected arguments (str, int)");
+    return NULL;
+  }
+
+  if (self->c_data == NULL) RET_ERR("data is not initialized", NULL);
+  ret = alpm_db_set_pkgreason(self->c_data, pkgname, reason);
+
+  if (ret == -1) RET_ERR("failed setting install reason", NULL);
+  Py_RETURN_NONE;
+}
+
 static PyObject* pyalpm_db_readgrp(PyObject* rawself, PyObject* args) {
   AlpmDB* self = (AlpmDB*)rawself;
   char *grpname;
@@ -151,6 +206,10 @@ static struct PyMethodDef db_methods[] = {
     "get contents of a group\n"
     "args: a group name (string)\n"
     "returns: a tuple (group name, list of packages)" },
+  { "set_pkgreason", pyalpm_db_set_pkgreason, METH_VARARGS,
+    "set install reason for a package\n"
+    "args: a package name (string), a reason (PKG_REASON_DEPEND, PKG_REASON_EXPLICIT)\n"
+    "returns: None" },
   { NULL },
 };
 
@@ -164,7 +223,7 @@ struct PyGetSetDef db_getset[] = {
   { NULL }
 };
 
-PyTypeObject AlpmDBType = {
+static PyTypeObject AlpmDBType = {
   PyVarObject_HEAD_INIT(NULL, 0)
   "alpm.DB",             /*tp_name*/
   sizeof(AlpmDB),        /*tp_basicsize*/
@@ -225,7 +284,7 @@ PyObject *pyalpm_db_from_pmdb(void* data) {
 
 /** non-class methods */
 PyObject* pyalpm_find_grp_pkgs(PyObject* self, PyObject *args) {
-  PyObject *dbs, *db_iter, *db;
+  PyObject *dbs;
   char *grpname;
   alpm_list_t *db_list = NULL;
   alpm_list_t *pkg_list;
@@ -236,25 +295,10 @@ PyObject* pyalpm_find_grp_pkgs(PyObject* self, PyObject *args) {
     return NULL;
   }
 
-  db_iter = PyObject_GetIter(dbs);
-  if(db_iter == NULL) {
-    PyErr_SetString(PyExc_TypeError, "argument is not iterable");
+  int ret = pylist_db_to_alpmlist(dbs, &db_list);
+  if (ret == -1)
     return NULL;
-  }
 
-  while((db = PyIter_Next(db_iter))) {
-    if (!PyObject_IsInstance(db, (PyObject*)&AlpmDBType)) {
-      PyErr_SetString(PyExc_TypeError, "non-DB in sequence");
-      Py_DECREF(db);
-      Py_DECREF(db_iter);
-      return NULL;
-    }
-
-    db_list = alpm_list_add(db_list, ((AlpmDB*)db)->c_data);
-    Py_DECREF(db);
-  }
-
-  Py_DECREF(db_iter);
   pkg_list = alpm_find_grp_pkgs(db_list, grpname);
   result = alpmlist_to_pylist(pkg_list, pyalpm_package_from_pmpkg);
   alpm_list_free(db_list);
@@ -262,4 +306,29 @@ PyObject* pyalpm_find_grp_pkgs(PyObject* self, PyObject *args) {
   return result;
 }
 
+/** Finds an available upgrade for a package in a list of databases */
+PyObject* pyalpm_sync_newversion(PyObject *self, PyObject* args) {
+  PyObject *pkg;
+  PyObject *dbs;
+  alpm_list_t *db_list;
+  pmpkg_t *result;
+  if(!PyArg_ParseTuple(args, "OO", &pkg, &dbs)
+      || !PyAlpmPkg_Check(pkg)
+      || pylist_db_to_alpmlist(dbs, &db_list) == -1)
+  {
+    PyErr_SetString(PyExc_TypeError, "sync_newversion() takes a Package and a list of DBs");
+    return NULL;
+  }
+
+  pmpkg_t *rawpkg = pmpkg_from_pyalpm_pkg(pkg);
+  if (!rawpkg)
+    return NULL;
+
+  result = alpm_sync_newversion(rawpkg, db_list);
+  if (!result)
+    Py_RETURN_NONE;
+  else
+    return pyalpm_package_from_pmpkg(result);
+}
+
 /* vim: set ts=2 sw=2 et: */
